-----------------------------------------------------------------------------------------------
-- Client Lua Script for DistanceTravel
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- DistanceTravel Module Definition
-----------------------------------------------------------------------------------------------
local DistanceTravel = {} 

DistanceTravel.previousPosition = nil

DistanceTravel.VERSION = 2

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function DistanceTravel:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function DistanceTravel:Init()
    Apollo.RegisterAddon(self, false)
	self.timer = ApolloTimer.Create(3.000, true, "OnTimer", self)
end
 

-----------------------------------------------------------------------------------------------
-- DistanceTravel OnLoad
-----------------------------------------------------------------------------------------------
function DistanceTravel:OnLoad()
	self.config = Apollo.GetPackage("Gemini:DB-1.0").tPackage:New(self)
	
	if self.config.char.VERSION == nil then
		self.config.char.distance = nil
		self.config.char.VERSION = self.VERSION
	end
	
	if self.config.char.distance == nil then
		self.config.char.distance = 0
	end
	
	if AddCharacterInformationLine ~= nil then
		self.LineObject = AddCharacterInformationLine(
		"DistanceTravel",
		"Distance traveled",
		function() 
			local dist = self.config.char.distance
			local unit = "m"
			
			if dist > 1000 then
				dist = self:round(dist / 1000, 2)
				unit = "km"
			end
			
			return dist .. " " .. unit 
		end,
		nil, 
		nil,
		"This is the total distance traveled around Nexus.")
	else
		self.LineObject = nil
	end
end

-----------------------------------------------------------------------------------------------
-- DistanceTravel Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on timer
function DistanceTravel:OnTimer()
	if self.previousPosition == nil then
		if GameLib.GetPlayerUnit() ~= nil then
			self.previousPosition = GameLib.GetPlayerUnit():GetPosition()
			return
		end
	else
		if GameLib.GetPlayerUnit() == nil then
			self.previousPosition = nil
			return
		end
		
		local actualPosition = GameLib.GetPlayerUnit():GetPosition()
		
		local dist = self:GetDistance(self.previousPosition, actualPosition)
		
		if dist < 1000 then
			self.config.char.distance = self.config.char.distance + dist
			
			if self.LineObject ~= nil then
				self.LineObject:forceUpdate()
			else
				Print("<DistanceTravel> You need to install the addon 'CharacterInfoPlus' (http://www.curse.com/ws-addons/wildstar/222978-characterinfoplus) to visualize the new amount into the character window.")
			end
		end
		
		self.previousPosition = actualPosition
		return
	end
end

function DistanceTravel:GetDistance(pos1, pos2)
	local dZ = pos2.z - pos1.z
	local dY = pos2.y - pos1.y
	local dX = pos2.x - pos1.x
	
	local dist = math.sqrt((dZ*dZ)+(dY*dY)+(dX*dX))
	
	return self:round(dist, 2)
end

function DistanceTravel:round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

-----------------------------------------------------------------------------------------------
-- DistanceTravel Instance
-----------------------------------------------------------------------------------------------
local DistanceTravelInst = DistanceTravel:new()
DistanceTravelInst:Init()
